/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

// VolumeStatusApplyConfiguration represents a declarative configuration of the VolumeStatus type for use
// with apply.
//
// VolumeStatus represents the status of a mounted volume.
// At most one of its members must be specified.
type VolumeStatusApplyConfiguration struct {
	// image represents an OCI object (a container image or artifact) pulled and mounted on the kubelet's host machine.
	Image *ImageVolumeStatusApplyConfiguration `json:"image,omitempty"`
}

// VolumeStatusApplyConfiguration constructs a declarative configuration of the VolumeStatus type for use with
// apply.
func VolumeStatus() *VolumeStatusApplyConfiguration {
	return &VolumeStatusApplyConfiguration{}
}

// WithImage sets the Image field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Image field is set to the value of the last call.
func (b *VolumeStatusApplyConfiguration) WithImage(value *ImageVolumeStatusApplyConfiguration) *VolumeStatusApplyConfiguration {
	b.Image = value
	return b
}
