// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

// Package meta provides definition of core metadata resources.
package meta

import "github.com/cosi-project/runtime/pkg/resource"

// NamespaceName is the name of 'meta' namespace.
const NamespaceName resource.Namespace = "meta"

// Owner is the owner for the 'meta' objects.
const Owner resource.Owner = "meta"
