// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11-devel
// 	protoc        (unknown)
// source: time/time.proto

package time

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"

	common "github.com/siderolabs/talos/pkg/machinery/api/common"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The response message containing the ntp server
type TimeRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Server        string                 `protobuf:"bytes,1,opt,name=server,proto3" json:"server,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TimeRequest) Reset() {
	*x = TimeRequest{}
	mi := &file_time_time_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TimeRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TimeRequest) ProtoMessage() {}

func (x *TimeRequest) ProtoReflect() protoreflect.Message {
	mi := &file_time_time_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TimeRequest.ProtoReflect.Descriptor instead.
func (*TimeRequest) Descriptor() ([]byte, []int) {
	return file_time_time_proto_rawDescGZIP(), []int{0}
}

func (x *TimeRequest) GetServer() string {
	if x != nil {
		return x.Server
	}
	return ""
}

type Time struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Server        string                 `protobuf:"bytes,2,opt,name=server,proto3" json:"server,omitempty"`
	Localtime     *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=localtime,proto3" json:"localtime,omitempty"`
	Remotetime    *timestamppb.Timestamp `protobuf:"bytes,4,opt,name=remotetime,proto3" json:"remotetime,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Time) Reset() {
	*x = Time{}
	mi := &file_time_time_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Time) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Time) ProtoMessage() {}

func (x *Time) ProtoReflect() protoreflect.Message {
	mi := &file_time_time_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Time.ProtoReflect.Descriptor instead.
func (*Time) Descriptor() ([]byte, []int) {
	return file_time_time_proto_rawDescGZIP(), []int{1}
}

func (x *Time) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Time) GetServer() string {
	if x != nil {
		return x.Server
	}
	return ""
}

func (x *Time) GetLocaltime() *timestamppb.Timestamp {
	if x != nil {
		return x.Localtime
	}
	return nil
}

func (x *Time) GetRemotetime() *timestamppb.Timestamp {
	if x != nil {
		return x.Remotetime
	}
	return nil
}

// The response message containing the ntp server, time, and offset
type TimeResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Time                `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TimeResponse) Reset() {
	*x = TimeResponse{}
	mi := &file_time_time_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TimeResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TimeResponse) ProtoMessage() {}

func (x *TimeResponse) ProtoReflect() protoreflect.Message {
	mi := &file_time_time_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TimeResponse.ProtoReflect.Descriptor instead.
func (*TimeResponse) Descriptor() ([]byte, []int) {
	return file_time_time_proto_rawDescGZIP(), []int{2}
}

func (x *TimeResponse) GetMessages() []*Time {
	if x != nil {
		return x.Messages
	}
	return nil
}

var File_time_time_proto protoreflect.FileDescriptor

const file_time_time_proto_rawDesc = "" +
	"\n" +
	"\x0ftime/time.proto\x12\x04time\x1a\x13common/common.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"%\n" +
	"\vTimeRequest\x12\x16\n" +
	"\x06server\x18\x01 \x01(\tR\x06server\"\xc2\x01\n" +
	"\x04Time\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x16\n" +
	"\x06server\x18\x02 \x01(\tR\x06server\x128\n" +
	"\tlocaltime\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\tlocaltime\x12:\n" +
	"\n" +
	"remotetime\x18\x04 \x01(\v2\x1a.google.protobuf.TimestampR\n" +
	"remotetime\"6\n" +
	"\fTimeResponse\x12&\n" +
	"\bmessages\x18\x01 \x03(\v2\n" +
	".time.TimeR\bmessages2u\n" +
	"\vTimeService\x122\n" +
	"\x04Time\x12\x16.google.protobuf.Empty\x1a\x12.time.TimeResponse\x122\n" +
	"\tTimeCheck\x12\x11.time.TimeRequest\x1a\x12.time.TimeResponseBH\n" +
	"\x12dev.talos.api.timeZ2github.com/siderolabs/talos/pkg/machinery/api/timeb\x06proto3"

var (
	file_time_time_proto_rawDescOnce sync.Once
	file_time_time_proto_rawDescData []byte
)

func file_time_time_proto_rawDescGZIP() []byte {
	file_time_time_proto_rawDescOnce.Do(func() {
		file_time_time_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_time_time_proto_rawDesc), len(file_time_time_proto_rawDesc)))
	})
	return file_time_time_proto_rawDescData
}

var file_time_time_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_time_time_proto_goTypes = []any{
	(*TimeRequest)(nil),           // 0: time.TimeRequest
	(*Time)(nil),                  // 1: time.Time
	(*TimeResponse)(nil),          // 2: time.TimeResponse
	(*common.Metadata)(nil),       // 3: common.Metadata
	(*timestamppb.Timestamp)(nil), // 4: google.protobuf.Timestamp
	(*emptypb.Empty)(nil),         // 5: google.protobuf.Empty
}
var file_time_time_proto_depIdxs = []int32{
	3, // 0: time.Time.metadata:type_name -> common.Metadata
	4, // 1: time.Time.localtime:type_name -> google.protobuf.Timestamp
	4, // 2: time.Time.remotetime:type_name -> google.protobuf.Timestamp
	1, // 3: time.TimeResponse.messages:type_name -> time.Time
	5, // 4: time.TimeService.Time:input_type -> google.protobuf.Empty
	0, // 5: time.TimeService.TimeCheck:input_type -> time.TimeRequest
	2, // 6: time.TimeService.Time:output_type -> time.TimeResponse
	2, // 7: time.TimeService.TimeCheck:output_type -> time.TimeResponse
	6, // [6:8] is the sub-list for method output_type
	4, // [4:6] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_time_time_proto_init() }
func file_time_time_proto_init() {
	if File_time_time_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_time_time_proto_rawDesc), len(file_time_time_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_time_time_proto_goTypes,
		DependencyIndexes: file_time_time_proto_depIdxs,
		MessageInfos:      file_time_time_proto_msgTypes,
	}.Build()
	File_time_time_proto = out.File
	file_time_time_proto_goTypes = nil
	file_time_time_proto_depIdxs = nil
}
