// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v6.31.1
// source: v1alpha1/state.proto

package v1alpha1

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type EventType int32

const (
	EventType_CREATED      EventType = 0
	EventType_UPDATED      EventType = 1
	EventType_DESTROYED    EventType = 2
	EventType_BOOTSTRAPPED EventType = 3
	EventType_ERRORED      EventType = 4
	EventType_NOOP         EventType = 5
)

// Enum value maps for EventType.
var (
	EventType_name = map[int32]string{
		0: "CREATED",
		1: "UPDATED",
		2: "DESTROYED",
		3: "BOOTSTRAPPED",
		4: "ERRORED",
		5: "NOOP",
	}
	EventType_value = map[string]int32{
		"CREATED":      0,
		"UPDATED":      1,
		"DESTROYED":    2,
		"BOOTSTRAPPED": 3,
		"ERRORED":      4,
		"NOOP":         5,
	}
)

func (x EventType) Enum() *EventType {
	p := new(EventType)
	*p = x
	return p
}

func (x EventType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (EventType) Descriptor() protoreflect.EnumDescriptor {
	return file_v1alpha1_state_proto_enumTypes[0].Descriptor()
}

func (EventType) Type() protoreflect.EnumType {
	return &file_v1alpha1_state_proto_enumTypes[0]
}

func (x EventType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use EventType.Descriptor instead.
func (EventType) EnumDescriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{0}
}

// Event is emitted when resource changes.
type Event struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Resource      *Resource              `protobuf:"bytes,1,opt,name=resource,proto3" json:"resource,omitempty"`
	Old           *Resource              `protobuf:"bytes,3,opt,name=old,proto3" json:"old,omitempty"`
	Error         *string                `protobuf:"bytes,4,opt,name=error,proto3,oneof" json:"error,omitempty"`
	EventType     EventType              `protobuf:"varint,2,opt,name=event_type,json=eventType,proto3,enum=cosi.resource.EventType" json:"event_type,omitempty"`
	Bookmark      []byte                 `protobuf:"bytes,5,opt,name=bookmark,proto3,oneof" json:"bookmark,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Event) Reset() {
	*x = Event{}
	mi := &file_v1alpha1_state_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Event) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Event) ProtoMessage() {}

func (x *Event) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Event.ProtoReflect.Descriptor instead.
func (*Event) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{0}
}

func (x *Event) GetResource() *Resource {
	if x != nil {
		return x.Resource
	}
	return nil
}

func (x *Event) GetOld() *Resource {
	if x != nil {
		return x.Old
	}
	return nil
}

func (x *Event) GetError() string {
	if x != nil && x.Error != nil {
		return *x.Error
	}
	return ""
}

func (x *Event) GetEventType() EventType {
	if x != nil {
		return x.EventType
	}
	return EventType_CREATED
}

func (x *Event) GetBookmark() []byte {
	if x != nil {
		return x.Bookmark
	}
	return nil
}

type GetRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Namespace     string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	Type          string                 `protobuf:"bytes,2,opt,name=type,proto3" json:"type,omitempty"`
	Id            string                 `protobuf:"bytes,3,opt,name=id,proto3" json:"id,omitempty"`
	Options       *GetOptions            `protobuf:"bytes,4,opt,name=options,proto3" json:"options,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetRequest) Reset() {
	*x = GetRequest{}
	mi := &file_v1alpha1_state_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetRequest) ProtoMessage() {}

func (x *GetRequest) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetRequest.ProtoReflect.Descriptor instead.
func (*GetRequest) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{1}
}

func (x *GetRequest) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *GetRequest) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

func (x *GetRequest) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *GetRequest) GetOptions() *GetOptions {
	if x != nil {
		return x.Options
	}
	return nil
}

type GetOptions struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetOptions) Reset() {
	*x = GetOptions{}
	mi := &file_v1alpha1_state_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetOptions) ProtoMessage() {}

func (x *GetOptions) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetOptions.ProtoReflect.Descriptor instead.
func (*GetOptions) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{2}
}

type GetResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Resource      *Resource              `protobuf:"bytes,1,opt,name=resource,proto3" json:"resource,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetResponse) Reset() {
	*x = GetResponse{}
	mi := &file_v1alpha1_state_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetResponse) ProtoMessage() {}

func (x *GetResponse) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetResponse.ProtoReflect.Descriptor instead.
func (*GetResponse) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{3}
}

func (x *GetResponse) GetResource() *Resource {
	if x != nil {
		return x.Resource
	}
	return nil
}

type ListRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Namespace     string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	Type          string                 `protobuf:"bytes,2,opt,name=type,proto3" json:"type,omitempty"`
	Options       *ListOptions           `protobuf:"bytes,3,opt,name=options,proto3" json:"options,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListRequest) Reset() {
	*x = ListRequest{}
	mi := &file_v1alpha1_state_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListRequest) ProtoMessage() {}

func (x *ListRequest) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListRequest.ProtoReflect.Descriptor instead.
func (*ListRequest) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{4}
}

func (x *ListRequest) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *ListRequest) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

func (x *ListRequest) GetOptions() *ListOptions {
	if x != nil {
		return x.Options
	}
	return nil
}

type ListOptions struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	LabelQuery    []*LabelQuery          `protobuf:"bytes,1,rep,name=label_query,json=labelQuery,proto3" json:"label_query,omitempty"`
	IdQuery       *IDQuery               `protobuf:"bytes,2,opt,name=id_query,json=idQuery,proto3" json:"id_query,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListOptions) Reset() {
	*x = ListOptions{}
	mi := &file_v1alpha1_state_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListOptions) ProtoMessage() {}

func (x *ListOptions) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListOptions.ProtoReflect.Descriptor instead.
func (*ListOptions) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{5}
}

func (x *ListOptions) GetLabelQuery() []*LabelQuery {
	if x != nil {
		return x.LabelQuery
	}
	return nil
}

func (x *ListOptions) GetIdQuery() *IDQuery {
	if x != nil {
		return x.IdQuery
	}
	return nil
}

type ListResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Resource      *Resource              `protobuf:"bytes,1,opt,name=resource,proto3" json:"resource,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListResponse) Reset() {
	*x = ListResponse{}
	mi := &file_v1alpha1_state_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListResponse) ProtoMessage() {}

func (x *ListResponse) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListResponse.ProtoReflect.Descriptor instead.
func (*ListResponse) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{6}
}

func (x *ListResponse) GetResource() *Resource {
	if x != nil {
		return x.Resource
	}
	return nil
}

type CreateRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Resource      *Resource              `protobuf:"bytes,1,opt,name=resource,proto3" json:"resource,omitempty"`
	Options       *CreateOptions         `protobuf:"bytes,2,opt,name=options,proto3" json:"options,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateRequest) Reset() {
	*x = CreateRequest{}
	mi := &file_v1alpha1_state_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateRequest) ProtoMessage() {}

func (x *CreateRequest) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateRequest.ProtoReflect.Descriptor instead.
func (*CreateRequest) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{7}
}

func (x *CreateRequest) GetResource() *Resource {
	if x != nil {
		return x.Resource
	}
	return nil
}

func (x *CreateRequest) GetOptions() *CreateOptions {
	if x != nil {
		return x.Options
	}
	return nil
}

type CreateOptions struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Owner         string                 `protobuf:"bytes,1,opt,name=owner,proto3" json:"owner,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateOptions) Reset() {
	*x = CreateOptions{}
	mi := &file_v1alpha1_state_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateOptions) ProtoMessage() {}

func (x *CreateOptions) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateOptions.ProtoReflect.Descriptor instead.
func (*CreateOptions) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{8}
}

func (x *CreateOptions) GetOwner() string {
	if x != nil {
		return x.Owner
	}
	return ""
}

type CreateResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Resource      *Resource              `protobuf:"bytes,1,opt,name=resource,proto3" json:"resource,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateResponse) Reset() {
	*x = CreateResponse{}
	mi := &file_v1alpha1_state_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateResponse) ProtoMessage() {}

func (x *CreateResponse) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateResponse.ProtoReflect.Descriptor instead.
func (*CreateResponse) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{9}
}

func (x *CreateResponse) GetResource() *Resource {
	if x != nil {
		return x.Resource
	}
	return nil
}

type UpdateRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	NewResource   *Resource              `protobuf:"bytes,2,opt,name=new_resource,json=newResource,proto3" json:"new_resource,omitempty"`
	Options       *UpdateOptions         `protobuf:"bytes,3,opt,name=options,proto3" json:"options,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateRequest) Reset() {
	*x = UpdateRequest{}
	mi := &file_v1alpha1_state_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateRequest) ProtoMessage() {}

func (x *UpdateRequest) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateRequest.ProtoReflect.Descriptor instead.
func (*UpdateRequest) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{10}
}

func (x *UpdateRequest) GetNewResource() *Resource {
	if x != nil {
		return x.NewResource
	}
	return nil
}

func (x *UpdateRequest) GetOptions() *UpdateOptions {
	if x != nil {
		return x.Options
	}
	return nil
}

type UpdateOptions struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Owner         string                 `protobuf:"bytes,1,opt,name=owner,proto3" json:"owner,omitempty"`
	ExpectedPhase *string                `protobuf:"bytes,2,opt,name=expected_phase,json=expectedPhase,proto3,oneof" json:"expected_phase,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateOptions) Reset() {
	*x = UpdateOptions{}
	mi := &file_v1alpha1_state_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateOptions) ProtoMessage() {}

func (x *UpdateOptions) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateOptions.ProtoReflect.Descriptor instead.
func (*UpdateOptions) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{11}
}

func (x *UpdateOptions) GetOwner() string {
	if x != nil {
		return x.Owner
	}
	return ""
}

func (x *UpdateOptions) GetExpectedPhase() string {
	if x != nil && x.ExpectedPhase != nil {
		return *x.ExpectedPhase
	}
	return ""
}

type UpdateResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Resource      *Resource              `protobuf:"bytes,1,opt,name=resource,proto3" json:"resource,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateResponse) Reset() {
	*x = UpdateResponse{}
	mi := &file_v1alpha1_state_proto_msgTypes[12]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateResponse) ProtoMessage() {}

func (x *UpdateResponse) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[12]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateResponse.ProtoReflect.Descriptor instead.
func (*UpdateResponse) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{12}
}

func (x *UpdateResponse) GetResource() *Resource {
	if x != nil {
		return x.Resource
	}
	return nil
}

type DestroyRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Namespace     string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	Type          string                 `protobuf:"bytes,2,opt,name=type,proto3" json:"type,omitempty"`
	Id            string                 `protobuf:"bytes,3,opt,name=id,proto3" json:"id,omitempty"`
	Options       *DestroyOptions        `protobuf:"bytes,4,opt,name=options,proto3" json:"options,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DestroyRequest) Reset() {
	*x = DestroyRequest{}
	mi := &file_v1alpha1_state_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DestroyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DestroyRequest) ProtoMessage() {}

func (x *DestroyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DestroyRequest.ProtoReflect.Descriptor instead.
func (*DestroyRequest) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{13}
}

func (x *DestroyRequest) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *DestroyRequest) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

func (x *DestroyRequest) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *DestroyRequest) GetOptions() *DestroyOptions {
	if x != nil {
		return x.Options
	}
	return nil
}

type DestroyOptions struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Owner         string                 `protobuf:"bytes,1,opt,name=owner,proto3" json:"owner,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DestroyOptions) Reset() {
	*x = DestroyOptions{}
	mi := &file_v1alpha1_state_proto_msgTypes[14]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DestroyOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DestroyOptions) ProtoMessage() {}

func (x *DestroyOptions) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[14]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DestroyOptions.ProtoReflect.Descriptor instead.
func (*DestroyOptions) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{14}
}

func (x *DestroyOptions) GetOwner() string {
	if x != nil {
		return x.Owner
	}
	return ""
}

type DestroyResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DestroyResponse) Reset() {
	*x = DestroyResponse{}
	mi := &file_v1alpha1_state_proto_msgTypes[15]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DestroyResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DestroyResponse) ProtoMessage() {}

func (x *DestroyResponse) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[15]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DestroyResponse.ProtoReflect.Descriptor instead.
func (*DestroyResponse) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{15}
}

type WatchRequest struct {
	state     protoimpl.MessageState `protogen:"open.v1"`
	Namespace string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	Type      string                 `protobuf:"bytes,2,opt,name=type,proto3" json:"type,omitempty"`
	Id        *string                `protobuf:"bytes,3,opt,name=id,proto3,oneof" json:"id,omitempty"`
	Options   *WatchOptions          `protobuf:"bytes,4,opt,name=options,proto3" json:"options,omitempty"`
	// Supported API versions:
	// 0 (not set): event types Created,Updated,Deleted
	// 1: additional event types Bootstrapped,Errored
	ApiVersion    int32 `protobuf:"varint,5,opt,name=api_version,json=apiVersion,proto3" json:"api_version,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *WatchRequest) Reset() {
	*x = WatchRequest{}
	mi := &file_v1alpha1_state_proto_msgTypes[16]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *WatchRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*WatchRequest) ProtoMessage() {}

func (x *WatchRequest) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[16]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use WatchRequest.ProtoReflect.Descriptor instead.
func (*WatchRequest) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{16}
}

func (x *WatchRequest) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *WatchRequest) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

func (x *WatchRequest) GetId() string {
	if x != nil && x.Id != nil {
		return *x.Id
	}
	return ""
}

func (x *WatchRequest) GetOptions() *WatchOptions {
	if x != nil {
		return x.Options
	}
	return nil
}

func (x *WatchRequest) GetApiVersion() int32 {
	if x != nil {
		return x.ApiVersion
	}
	return 0
}

type WatchOptions struct {
	state             protoimpl.MessageState `protogen:"open.v1"`
	BootstrapContents bool                   `protobuf:"varint,1,opt,name=bootstrap_contents,json=bootstrapContents,proto3" json:"bootstrap_contents,omitempty"`
	TailEvents        int32                  `protobuf:"varint,2,opt,name=tail_events,json=tailEvents,proto3" json:"tail_events,omitempty"`
	LabelQuery        []*LabelQuery          `protobuf:"bytes,3,rep,name=label_query,json=labelQuery,proto3" json:"label_query,omitempty"`
	IdQuery           *IDQuery               `protobuf:"bytes,4,opt,name=id_query,json=idQuery,proto3" json:"id_query,omitempty"`
	Aggregated        bool                   `protobuf:"varint,5,opt,name=aggregated,proto3" json:"aggregated,omitempty"`
	StartFromBookmark []byte                 `protobuf:"bytes,6,opt,name=start_from_bookmark,json=startFromBookmark,proto3,oneof" json:"start_from_bookmark,omitempty"`
	BootstrapBookmark bool                   `protobuf:"varint,7,opt,name=bootstrap_bookmark,json=bootstrapBookmark,proto3" json:"bootstrap_bookmark,omitempty"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *WatchOptions) Reset() {
	*x = WatchOptions{}
	mi := &file_v1alpha1_state_proto_msgTypes[17]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *WatchOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*WatchOptions) ProtoMessage() {}

func (x *WatchOptions) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[17]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use WatchOptions.ProtoReflect.Descriptor instead.
func (*WatchOptions) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{17}
}

func (x *WatchOptions) GetBootstrapContents() bool {
	if x != nil {
		return x.BootstrapContents
	}
	return false
}

func (x *WatchOptions) GetTailEvents() int32 {
	if x != nil {
		return x.TailEvents
	}
	return 0
}

func (x *WatchOptions) GetLabelQuery() []*LabelQuery {
	if x != nil {
		return x.LabelQuery
	}
	return nil
}

func (x *WatchOptions) GetIdQuery() *IDQuery {
	if x != nil {
		return x.IdQuery
	}
	return nil
}

func (x *WatchOptions) GetAggregated() bool {
	if x != nil {
		return x.Aggregated
	}
	return false
}

func (x *WatchOptions) GetStartFromBookmark() []byte {
	if x != nil {
		return x.StartFromBookmark
	}
	return nil
}

func (x *WatchOptions) GetBootstrapBookmark() bool {
	if x != nil {
		return x.BootstrapBookmark
	}
	return false
}

type WatchResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Event         []*Event               `protobuf:"bytes,1,rep,name=event,proto3" json:"event,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *WatchResponse) Reset() {
	*x = WatchResponse{}
	mi := &file_v1alpha1_state_proto_msgTypes[18]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *WatchResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*WatchResponse) ProtoMessage() {}

func (x *WatchResponse) ProtoReflect() protoreflect.Message {
	mi := &file_v1alpha1_state_proto_msgTypes[18]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use WatchResponse.ProtoReflect.Descriptor instead.
func (*WatchResponse) Descriptor() ([]byte, []int) {
	return file_v1alpha1_state_proto_rawDescGZIP(), []int{18}
}

func (x *WatchResponse) GetEvent() []*Event {
	if x != nil {
		return x.Event
	}
	return nil
}

var File_v1alpha1_state_proto protoreflect.FileDescriptor

const file_v1alpha1_state_proto_rawDesc = "" +
	"\n" +
	"\x14v1alpha1/state.proto\x12\rcosi.resource\x1a\x17v1alpha1/resource.proto\"\xf3\x01\n" +
	"\x05Event\x123\n" +
	"\bresource\x18\x01 \x01(\v2\x17.cosi.resource.ResourceR\bresource\x12)\n" +
	"\x03old\x18\x03 \x01(\v2\x17.cosi.resource.ResourceR\x03old\x12\x19\n" +
	"\x05error\x18\x04 \x01(\tH\x00R\x05error\x88\x01\x01\x127\n" +
	"\n" +
	"event_type\x18\x02 \x01(\x0e2\x18.cosi.resource.EventTypeR\teventType\x12\x1f\n" +
	"\bbookmark\x18\x05 \x01(\fH\x01R\bbookmark\x88\x01\x01B\b\n" +
	"\x06_errorB\v\n" +
	"\t_bookmark\"\x83\x01\n" +
	"\n" +
	"GetRequest\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x12\n" +
	"\x04type\x18\x02 \x01(\tR\x04type\x12\x0e\n" +
	"\x02id\x18\x03 \x01(\tR\x02id\x123\n" +
	"\aoptions\x18\x04 \x01(\v2\x19.cosi.resource.GetOptionsR\aoptions\"\f\n" +
	"\n" +
	"GetOptions\"B\n" +
	"\vGetResponse\x123\n" +
	"\bresource\x18\x01 \x01(\v2\x17.cosi.resource.ResourceR\bresource\"u\n" +
	"\vListRequest\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x12\n" +
	"\x04type\x18\x02 \x01(\tR\x04type\x124\n" +
	"\aoptions\x18\x03 \x01(\v2\x1a.cosi.resource.ListOptionsR\aoptions\"|\n" +
	"\vListOptions\x12:\n" +
	"\vlabel_query\x18\x01 \x03(\v2\x19.cosi.resource.LabelQueryR\n" +
	"labelQuery\x121\n" +
	"\bid_query\x18\x02 \x01(\v2\x16.cosi.resource.IDQueryR\aidQuery\"C\n" +
	"\fListResponse\x123\n" +
	"\bresource\x18\x01 \x01(\v2\x17.cosi.resource.ResourceR\bresource\"|\n" +
	"\rCreateRequest\x123\n" +
	"\bresource\x18\x01 \x01(\v2\x17.cosi.resource.ResourceR\bresource\x126\n" +
	"\aoptions\x18\x02 \x01(\v2\x1c.cosi.resource.CreateOptionsR\aoptions\"%\n" +
	"\rCreateOptions\x12\x14\n" +
	"\x05owner\x18\x01 \x01(\tR\x05owner\"E\n" +
	"\x0eCreateResponse\x123\n" +
	"\bresource\x18\x01 \x01(\v2\x17.cosi.resource.ResourceR\bresource\"\x9a\x01\n" +
	"\rUpdateRequest\x12:\n" +
	"\fnew_resource\x18\x02 \x01(\v2\x17.cosi.resource.ResourceR\vnewResource\x126\n" +
	"\aoptions\x18\x03 \x01(\v2\x1c.cosi.resource.UpdateOptionsR\aoptionsJ\x04\b\x01\x10\x02R\x0fcurrent_version\"d\n" +
	"\rUpdateOptions\x12\x14\n" +
	"\x05owner\x18\x01 \x01(\tR\x05owner\x12*\n" +
	"\x0eexpected_phase\x18\x02 \x01(\tH\x00R\rexpectedPhase\x88\x01\x01B\x11\n" +
	"\x0f_expected_phase\"E\n" +
	"\x0eUpdateResponse\x123\n" +
	"\bresource\x18\x01 \x01(\v2\x17.cosi.resource.ResourceR\bresource\"\x8b\x01\n" +
	"\x0eDestroyRequest\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x12\n" +
	"\x04type\x18\x02 \x01(\tR\x04type\x12\x0e\n" +
	"\x02id\x18\x03 \x01(\tR\x02id\x127\n" +
	"\aoptions\x18\x04 \x01(\v2\x1d.cosi.resource.DestroyOptionsR\aoptions\"&\n" +
	"\x0eDestroyOptions\x12\x14\n" +
	"\x05owner\x18\x01 \x01(\tR\x05owner\"\x11\n" +
	"\x0fDestroyResponse\"\xb4\x01\n" +
	"\fWatchRequest\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x12\n" +
	"\x04type\x18\x02 \x01(\tR\x04type\x12\x13\n" +
	"\x02id\x18\x03 \x01(\tH\x00R\x02id\x88\x01\x01\x125\n" +
	"\aoptions\x18\x04 \x01(\v2\x1b.cosi.resource.WatchOptionsR\aoptions\x12\x1f\n" +
	"\vapi_version\x18\x05 \x01(\x05R\n" +
	"apiVersionB\x05\n" +
	"\x03_id\"\xe9\x02\n" +
	"\fWatchOptions\x12-\n" +
	"\x12bootstrap_contents\x18\x01 \x01(\bR\x11bootstrapContents\x12\x1f\n" +
	"\vtail_events\x18\x02 \x01(\x05R\n" +
	"tailEvents\x12:\n" +
	"\vlabel_query\x18\x03 \x03(\v2\x19.cosi.resource.LabelQueryR\n" +
	"labelQuery\x121\n" +
	"\bid_query\x18\x04 \x01(\v2\x16.cosi.resource.IDQueryR\aidQuery\x12\x1e\n" +
	"\n" +
	"aggregated\x18\x05 \x01(\bR\n" +
	"aggregated\x123\n" +
	"\x13start_from_bookmark\x18\x06 \x01(\fH\x00R\x11startFromBookmark\x88\x01\x01\x12-\n" +
	"\x12bootstrap_bookmark\x18\a \x01(\bR\x11bootstrapBookmarkB\x16\n" +
	"\x14_start_from_bookmark\";\n" +
	"\rWatchResponse\x12*\n" +
	"\x05event\x18\x01 \x03(\v2\x14.cosi.resource.EventR\x05event*]\n" +
	"\tEventType\x12\v\n" +
	"\aCREATED\x10\x00\x12\v\n" +
	"\aUPDATED\x10\x01\x12\r\n" +
	"\tDESTROYED\x10\x02\x12\x10\n" +
	"\fBOOTSTRAPPED\x10\x03\x12\v\n" +
	"\aERRORED\x10\x04\x12\b\n" +
	"\x04NOOP\x10\x052\xa6\x03\n" +
	"\x05State\x12<\n" +
	"\x03Get\x12\x19.cosi.resource.GetRequest\x1a\x1a.cosi.resource.GetResponse\x12A\n" +
	"\x04List\x12\x1a.cosi.resource.ListRequest\x1a\x1b.cosi.resource.ListResponse0\x01\x12E\n" +
	"\x06Create\x12\x1c.cosi.resource.CreateRequest\x1a\x1d.cosi.resource.CreateResponse\x12E\n" +
	"\x06Update\x12\x1c.cosi.resource.UpdateRequest\x1a\x1d.cosi.resource.UpdateResponse\x12H\n" +
	"\aDestroy\x12\x1d.cosi.resource.DestroyRequest\x1a\x1e.cosi.resource.DestroyResponse\x12D\n" +
	"\x05Watch\x12\x1b.cosi.resource.WatchRequest\x1a\x1c.cosi.resource.WatchResponse0\x01B.Z,github.com/cosi-project/runtime/api/v1alpha1b\x06proto3"

var (
	file_v1alpha1_state_proto_rawDescOnce sync.Once
	file_v1alpha1_state_proto_rawDescData []byte
)

func file_v1alpha1_state_proto_rawDescGZIP() []byte {
	file_v1alpha1_state_proto_rawDescOnce.Do(func() {
		file_v1alpha1_state_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_v1alpha1_state_proto_rawDesc), len(file_v1alpha1_state_proto_rawDesc)))
	})
	return file_v1alpha1_state_proto_rawDescData
}

var file_v1alpha1_state_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_v1alpha1_state_proto_msgTypes = make([]protoimpl.MessageInfo, 19)
var file_v1alpha1_state_proto_goTypes = []any{
	(EventType)(0),          // 0: cosi.resource.EventType
	(*Event)(nil),           // 1: cosi.resource.Event
	(*GetRequest)(nil),      // 2: cosi.resource.GetRequest
	(*GetOptions)(nil),      // 3: cosi.resource.GetOptions
	(*GetResponse)(nil),     // 4: cosi.resource.GetResponse
	(*ListRequest)(nil),     // 5: cosi.resource.ListRequest
	(*ListOptions)(nil),     // 6: cosi.resource.ListOptions
	(*ListResponse)(nil),    // 7: cosi.resource.ListResponse
	(*CreateRequest)(nil),   // 8: cosi.resource.CreateRequest
	(*CreateOptions)(nil),   // 9: cosi.resource.CreateOptions
	(*CreateResponse)(nil),  // 10: cosi.resource.CreateResponse
	(*UpdateRequest)(nil),   // 11: cosi.resource.UpdateRequest
	(*UpdateOptions)(nil),   // 12: cosi.resource.UpdateOptions
	(*UpdateResponse)(nil),  // 13: cosi.resource.UpdateResponse
	(*DestroyRequest)(nil),  // 14: cosi.resource.DestroyRequest
	(*DestroyOptions)(nil),  // 15: cosi.resource.DestroyOptions
	(*DestroyResponse)(nil), // 16: cosi.resource.DestroyResponse
	(*WatchRequest)(nil),    // 17: cosi.resource.WatchRequest
	(*WatchOptions)(nil),    // 18: cosi.resource.WatchOptions
	(*WatchResponse)(nil),   // 19: cosi.resource.WatchResponse
	(*Resource)(nil),        // 20: cosi.resource.Resource
	(*LabelQuery)(nil),      // 21: cosi.resource.LabelQuery
	(*IDQuery)(nil),         // 22: cosi.resource.IDQuery
}
var file_v1alpha1_state_proto_depIdxs = []int32{
	20, // 0: cosi.resource.Event.resource:type_name -> cosi.resource.Resource
	20, // 1: cosi.resource.Event.old:type_name -> cosi.resource.Resource
	0,  // 2: cosi.resource.Event.event_type:type_name -> cosi.resource.EventType
	3,  // 3: cosi.resource.GetRequest.options:type_name -> cosi.resource.GetOptions
	20, // 4: cosi.resource.GetResponse.resource:type_name -> cosi.resource.Resource
	6,  // 5: cosi.resource.ListRequest.options:type_name -> cosi.resource.ListOptions
	21, // 6: cosi.resource.ListOptions.label_query:type_name -> cosi.resource.LabelQuery
	22, // 7: cosi.resource.ListOptions.id_query:type_name -> cosi.resource.IDQuery
	20, // 8: cosi.resource.ListResponse.resource:type_name -> cosi.resource.Resource
	20, // 9: cosi.resource.CreateRequest.resource:type_name -> cosi.resource.Resource
	9,  // 10: cosi.resource.CreateRequest.options:type_name -> cosi.resource.CreateOptions
	20, // 11: cosi.resource.CreateResponse.resource:type_name -> cosi.resource.Resource
	20, // 12: cosi.resource.UpdateRequest.new_resource:type_name -> cosi.resource.Resource
	12, // 13: cosi.resource.UpdateRequest.options:type_name -> cosi.resource.UpdateOptions
	20, // 14: cosi.resource.UpdateResponse.resource:type_name -> cosi.resource.Resource
	15, // 15: cosi.resource.DestroyRequest.options:type_name -> cosi.resource.DestroyOptions
	18, // 16: cosi.resource.WatchRequest.options:type_name -> cosi.resource.WatchOptions
	21, // 17: cosi.resource.WatchOptions.label_query:type_name -> cosi.resource.LabelQuery
	22, // 18: cosi.resource.WatchOptions.id_query:type_name -> cosi.resource.IDQuery
	1,  // 19: cosi.resource.WatchResponse.event:type_name -> cosi.resource.Event
	2,  // 20: cosi.resource.State.Get:input_type -> cosi.resource.GetRequest
	5,  // 21: cosi.resource.State.List:input_type -> cosi.resource.ListRequest
	8,  // 22: cosi.resource.State.Create:input_type -> cosi.resource.CreateRequest
	11, // 23: cosi.resource.State.Update:input_type -> cosi.resource.UpdateRequest
	14, // 24: cosi.resource.State.Destroy:input_type -> cosi.resource.DestroyRequest
	17, // 25: cosi.resource.State.Watch:input_type -> cosi.resource.WatchRequest
	4,  // 26: cosi.resource.State.Get:output_type -> cosi.resource.GetResponse
	7,  // 27: cosi.resource.State.List:output_type -> cosi.resource.ListResponse
	10, // 28: cosi.resource.State.Create:output_type -> cosi.resource.CreateResponse
	13, // 29: cosi.resource.State.Update:output_type -> cosi.resource.UpdateResponse
	16, // 30: cosi.resource.State.Destroy:output_type -> cosi.resource.DestroyResponse
	19, // 31: cosi.resource.State.Watch:output_type -> cosi.resource.WatchResponse
	26, // [26:32] is the sub-list for method output_type
	20, // [20:26] is the sub-list for method input_type
	20, // [20:20] is the sub-list for extension type_name
	20, // [20:20] is the sub-list for extension extendee
	0,  // [0:20] is the sub-list for field type_name
}

func init() { file_v1alpha1_state_proto_init() }
func file_v1alpha1_state_proto_init() {
	if File_v1alpha1_state_proto != nil {
		return
	}
	file_v1alpha1_resource_proto_init()
	file_v1alpha1_state_proto_msgTypes[0].OneofWrappers = []any{}
	file_v1alpha1_state_proto_msgTypes[11].OneofWrappers = []any{}
	file_v1alpha1_state_proto_msgTypes[16].OneofWrappers = []any{}
	file_v1alpha1_state_proto_msgTypes[17].OneofWrappers = []any{}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_v1alpha1_state_proto_rawDesc), len(file_v1alpha1_state_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   19,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_v1alpha1_state_proto_goTypes,
		DependencyIndexes: file_v1alpha1_state_proto_depIdxs,
		EnumInfos:         file_v1alpha1_state_proto_enumTypes,
		MessageInfos:      file_v1alpha1_state_proto_msgTypes,
	}.Build()
	File_v1alpha1_state_proto = out.File
	file_v1alpha1_state_proto_goTypes = nil
	file_v1alpha1_state_proto_depIdxs = nil
}
