// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v5.29.4
// source: auth/auth.proto

package auth

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type PublicKey struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	PgpData       []byte                 `protobuf:"bytes,1,opt,name=pgp_data,json=pgpData,proto3" json:"pgp_data,omitempty"`
	WebauthnData  []byte                 `protobuf:"bytes,2,opt,name=webauthn_data,json=webauthnData,proto3" json:"webauthn_data,omitempty"`
	PlainKey      *PublicKey_Plain       `protobuf:"bytes,3,opt,name=plain_key,json=plainKey,proto3" json:"plain_key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PublicKey) Reset() {
	*x = PublicKey{}
	mi := &file_auth_auth_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PublicKey) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PublicKey) ProtoMessage() {}

func (x *PublicKey) ProtoReflect() protoreflect.Message {
	mi := &file_auth_auth_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PublicKey.ProtoReflect.Descriptor instead.
func (*PublicKey) Descriptor() ([]byte, []int) {
	return file_auth_auth_proto_rawDescGZIP(), []int{0}
}

func (x *PublicKey) GetPgpData() []byte {
	if x != nil {
		return x.PgpData
	}
	return nil
}

func (x *PublicKey) GetWebauthnData() []byte {
	if x != nil {
		return x.WebauthnData
	}
	return nil
}

func (x *PublicKey) GetPlainKey() *PublicKey_Plain {
	if x != nil {
		return x.PlainKey
	}
	return nil
}

type Identity struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Email         string                 `protobuf:"bytes,1,opt,name=email,proto3" json:"email,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Identity) Reset() {
	*x = Identity{}
	mi := &file_auth_auth_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Identity) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Identity) ProtoMessage() {}

func (x *Identity) ProtoReflect() protoreflect.Message {
	mi := &file_auth_auth_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Identity.ProtoReflect.Descriptor instead.
func (*Identity) Descriptor() ([]byte, []int) {
	return file_auth_auth_proto_rawDescGZIP(), []int{1}
}

func (x *Identity) GetEmail() string {
	if x != nil {
		return x.Email
	}
	return ""
}

type RegisterPublicKeyRequest struct {
	state     protoimpl.MessageState `protogen:"open.v1"`
	PublicKey *PublicKey             `protobuf:"bytes,1,opt,name=public_key,json=publicKey,proto3" json:"public_key,omitempty"`
	Identity  *Identity              `protobuf:"bytes,2,opt,name=identity,proto3" json:"identity,omitempty"`
	// The role of the public key. When skip_user_role is true, this field is ignored.
	Role string `protobuf:"bytes,5,opt,name=role,proto3" json:"role,omitempty"`
	// If true, the role field will be used to determine the role of the public key.
	SkipUserRole  bool `protobuf:"varint,6,opt,name=skip_user_role,json=skipUserRole,proto3" json:"skip_user_role,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RegisterPublicKeyRequest) Reset() {
	*x = RegisterPublicKeyRequest{}
	mi := &file_auth_auth_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RegisterPublicKeyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RegisterPublicKeyRequest) ProtoMessage() {}

func (x *RegisterPublicKeyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_auth_auth_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RegisterPublicKeyRequest.ProtoReflect.Descriptor instead.
func (*RegisterPublicKeyRequest) Descriptor() ([]byte, []int) {
	return file_auth_auth_proto_rawDescGZIP(), []int{2}
}

func (x *RegisterPublicKeyRequest) GetPublicKey() *PublicKey {
	if x != nil {
		return x.PublicKey
	}
	return nil
}

func (x *RegisterPublicKeyRequest) GetIdentity() *Identity {
	if x != nil {
		return x.Identity
	}
	return nil
}

func (x *RegisterPublicKeyRequest) GetRole() string {
	if x != nil {
		return x.Role
	}
	return ""
}

func (x *RegisterPublicKeyRequest) GetSkipUserRole() bool {
	if x != nil {
		return x.SkipUserRole
	}
	return false
}

type RegisterPublicKeyResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	LoginUrl      string                 `protobuf:"bytes,1,opt,name=login_url,json=loginUrl,proto3" json:"login_url,omitempty"`
	PublicKeyId   string                 `protobuf:"bytes,2,opt,name=public_key_id,json=publicKeyId,proto3" json:"public_key_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RegisterPublicKeyResponse) Reset() {
	*x = RegisterPublicKeyResponse{}
	mi := &file_auth_auth_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RegisterPublicKeyResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RegisterPublicKeyResponse) ProtoMessage() {}

func (x *RegisterPublicKeyResponse) ProtoReflect() protoreflect.Message {
	mi := &file_auth_auth_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RegisterPublicKeyResponse.ProtoReflect.Descriptor instead.
func (*RegisterPublicKeyResponse) Descriptor() ([]byte, []int) {
	return file_auth_auth_proto_rawDescGZIP(), []int{3}
}

func (x *RegisterPublicKeyResponse) GetLoginUrl() string {
	if x != nil {
		return x.LoginUrl
	}
	return ""
}

func (x *RegisterPublicKeyResponse) GetPublicKeyId() string {
	if x != nil {
		return x.PublicKeyId
	}
	return ""
}

type AwaitPublicKeyConfirmationRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	PublicKeyId   string                 `protobuf:"bytes,1,opt,name=public_key_id,json=publicKeyId,proto3" json:"public_key_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AwaitPublicKeyConfirmationRequest) Reset() {
	*x = AwaitPublicKeyConfirmationRequest{}
	mi := &file_auth_auth_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AwaitPublicKeyConfirmationRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AwaitPublicKeyConfirmationRequest) ProtoMessage() {}

func (x *AwaitPublicKeyConfirmationRequest) ProtoReflect() protoreflect.Message {
	mi := &file_auth_auth_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AwaitPublicKeyConfirmationRequest.ProtoReflect.Descriptor instead.
func (*AwaitPublicKeyConfirmationRequest) Descriptor() ([]byte, []int) {
	return file_auth_auth_proto_rawDescGZIP(), []int{4}
}

func (x *AwaitPublicKeyConfirmationRequest) GetPublicKeyId() string {
	if x != nil {
		return x.PublicKeyId
	}
	return ""
}

type ConfirmPublicKeyRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	PublicKeyId   string                 `protobuf:"bytes,1,opt,name=public_key_id,json=publicKeyId,proto3" json:"public_key_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ConfirmPublicKeyRequest) Reset() {
	*x = ConfirmPublicKeyRequest{}
	mi := &file_auth_auth_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ConfirmPublicKeyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ConfirmPublicKeyRequest) ProtoMessage() {}

func (x *ConfirmPublicKeyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_auth_auth_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ConfirmPublicKeyRequest.ProtoReflect.Descriptor instead.
func (*ConfirmPublicKeyRequest) Descriptor() ([]byte, []int) {
	return file_auth_auth_proto_rawDescGZIP(), []int{5}
}

func (x *ConfirmPublicKeyRequest) GetPublicKeyId() string {
	if x != nil {
		return x.PublicKeyId
	}
	return ""
}

type RevokePublicKeyRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	PublicKeyId   string                 `protobuf:"bytes,1,opt,name=public_key_id,json=publicKeyId,proto3" json:"public_key_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RevokePublicKeyRequest) Reset() {
	*x = RevokePublicKeyRequest{}
	mi := &file_auth_auth_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RevokePublicKeyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RevokePublicKeyRequest) ProtoMessage() {}

func (x *RevokePublicKeyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_auth_auth_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RevokePublicKeyRequest.ProtoReflect.Descriptor instead.
func (*RevokePublicKeyRequest) Descriptor() ([]byte, []int) {
	return file_auth_auth_proto_rawDescGZIP(), []int{6}
}

func (x *RevokePublicKeyRequest) GetPublicKeyId() string {
	if x != nil {
		return x.PublicKeyId
	}
	return ""
}

type PublicKey_Plain struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	KeyPem        string                 `protobuf:"bytes,1,opt,name=key_pem,json=keyPem,proto3" json:"key_pem,omitempty"`
	NotBefore     *timestamppb.Timestamp `protobuf:"bytes,2,opt,name=not_before,json=notBefore,proto3" json:"not_before,omitempty"`
	NotAfter      *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=not_after,json=notAfter,proto3" json:"not_after,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PublicKey_Plain) Reset() {
	*x = PublicKey_Plain{}
	mi := &file_auth_auth_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PublicKey_Plain) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PublicKey_Plain) ProtoMessage() {}

func (x *PublicKey_Plain) ProtoReflect() protoreflect.Message {
	mi := &file_auth_auth_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PublicKey_Plain.ProtoReflect.Descriptor instead.
func (*PublicKey_Plain) Descriptor() ([]byte, []int) {
	return file_auth_auth_proto_rawDescGZIP(), []int{0, 0}
}

func (x *PublicKey_Plain) GetKeyPem() string {
	if x != nil {
		return x.KeyPem
	}
	return ""
}

func (x *PublicKey_Plain) GetNotBefore() *timestamppb.Timestamp {
	if x != nil {
		return x.NotBefore
	}
	return nil
}

func (x *PublicKey_Plain) GetNotAfter() *timestamppb.Timestamp {
	if x != nil {
		return x.NotAfter
	}
	return nil
}

var File_auth_auth_proto protoreflect.FileDescriptor

const file_auth_auth_proto_rawDesc = "" +
	"\n" +
	"\x0fauth/auth.proto\x12\x04auth\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x96\x02\n" +
	"\tPublicKey\x12\x19\n" +
	"\bpgp_data\x18\x01 \x01(\fR\apgpData\x12#\n" +
	"\rwebauthn_data\x18\x02 \x01(\fR\fwebauthnData\x122\n" +
	"\tplain_key\x18\x03 \x01(\v2\x15.auth.PublicKey.PlainR\bplainKey\x1a\x94\x01\n" +
	"\x05Plain\x12\x17\n" +
	"\akey_pem\x18\x01 \x01(\tR\x06keyPem\x129\n" +
	"\n" +
	"not_before\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\tnotBefore\x127\n" +
	"\tnot_after\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\bnotAfter\" \n" +
	"\bIdentity\x12\x14\n" +
	"\x05email\x18\x01 \x01(\tR\x05email\"\xbc\x01\n" +
	"\x18RegisterPublicKeyRequest\x12.\n" +
	"\n" +
	"public_key\x18\x01 \x01(\v2\x0f.auth.PublicKeyR\tpublicKey\x12*\n" +
	"\bidentity\x18\x02 \x01(\v2\x0e.auth.IdentityR\bidentity\x12\x12\n" +
	"\x04role\x18\x05 \x01(\tR\x04role\x12$\n" +
	"\x0eskip_user_role\x18\x06 \x01(\bR\fskipUserRoleJ\x04\b\x03\x10\x04J\x04\b\x04\x10\x05\"\\\n" +
	"\x19RegisterPublicKeyResponse\x12\x1b\n" +
	"\tlogin_url\x18\x01 \x01(\tR\bloginUrl\x12\"\n" +
	"\rpublic_key_id\x18\x02 \x01(\tR\vpublicKeyId\"G\n" +
	"!AwaitPublicKeyConfirmationRequest\x12\"\n" +
	"\rpublic_key_id\x18\x01 \x01(\tR\vpublicKeyId\"=\n" +
	"\x17ConfirmPublicKeyRequest\x12\"\n" +
	"\rpublic_key_id\x18\x01 \x01(\tR\vpublicKeyId\"<\n" +
	"\x16RevokePublicKeyRequest\x12\"\n" +
	"\rpublic_key_id\x18\x01 \x01(\tR\vpublicKeyId2\xd6\x02\n" +
	"\vAuthService\x12T\n" +
	"\x11RegisterPublicKey\x12\x1e.auth.RegisterPublicKeyRequest\x1a\x1f.auth.RegisterPublicKeyResponse\x12]\n" +
	"\x1aAwaitPublicKeyConfirmation\x12'.auth.AwaitPublicKeyConfirmationRequest\x1a\x16.google.protobuf.Empty\x12I\n" +
	"\x10ConfirmPublicKey\x12\x1d.auth.ConfirmPublicKeyRequest\x1a\x16.google.protobuf.Empty\x12G\n" +
	"\x0fRevokePublicKey\x12\x1c.auth.RevokePublicKeyRequest\x1a\x16.google.protobuf.EmptyB1Z/github.com/siderolabs/go-api-signature/api/authb\x06proto3"

var (
	file_auth_auth_proto_rawDescOnce sync.Once
	file_auth_auth_proto_rawDescData []byte
)

func file_auth_auth_proto_rawDescGZIP() []byte {
	file_auth_auth_proto_rawDescOnce.Do(func() {
		file_auth_auth_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_auth_auth_proto_rawDesc), len(file_auth_auth_proto_rawDesc)))
	})
	return file_auth_auth_proto_rawDescData
}

var file_auth_auth_proto_msgTypes = make([]protoimpl.MessageInfo, 8)
var file_auth_auth_proto_goTypes = []any{
	(*PublicKey)(nil),                         // 0: auth.PublicKey
	(*Identity)(nil),                          // 1: auth.Identity
	(*RegisterPublicKeyRequest)(nil),          // 2: auth.RegisterPublicKeyRequest
	(*RegisterPublicKeyResponse)(nil),         // 3: auth.RegisterPublicKeyResponse
	(*AwaitPublicKeyConfirmationRequest)(nil), // 4: auth.AwaitPublicKeyConfirmationRequest
	(*ConfirmPublicKeyRequest)(nil),           // 5: auth.ConfirmPublicKeyRequest
	(*RevokePublicKeyRequest)(nil),            // 6: auth.RevokePublicKeyRequest
	(*PublicKey_Plain)(nil),                   // 7: auth.PublicKey.Plain
	(*timestamppb.Timestamp)(nil),             // 8: google.protobuf.Timestamp
	(*emptypb.Empty)(nil),                     // 9: google.protobuf.Empty
}
var file_auth_auth_proto_depIdxs = []int32{
	7, // 0: auth.PublicKey.plain_key:type_name -> auth.PublicKey.Plain
	0, // 1: auth.RegisterPublicKeyRequest.public_key:type_name -> auth.PublicKey
	1, // 2: auth.RegisterPublicKeyRequest.identity:type_name -> auth.Identity
	8, // 3: auth.PublicKey.Plain.not_before:type_name -> google.protobuf.Timestamp
	8, // 4: auth.PublicKey.Plain.not_after:type_name -> google.protobuf.Timestamp
	2, // 5: auth.AuthService.RegisterPublicKey:input_type -> auth.RegisterPublicKeyRequest
	4, // 6: auth.AuthService.AwaitPublicKeyConfirmation:input_type -> auth.AwaitPublicKeyConfirmationRequest
	5, // 7: auth.AuthService.ConfirmPublicKey:input_type -> auth.ConfirmPublicKeyRequest
	6, // 8: auth.AuthService.RevokePublicKey:input_type -> auth.RevokePublicKeyRequest
	3, // 9: auth.AuthService.RegisterPublicKey:output_type -> auth.RegisterPublicKeyResponse
	9, // 10: auth.AuthService.AwaitPublicKeyConfirmation:output_type -> google.protobuf.Empty
	9, // 11: auth.AuthService.ConfirmPublicKey:output_type -> google.protobuf.Empty
	9, // 12: auth.AuthService.RevokePublicKey:output_type -> google.protobuf.Empty
	9, // [9:13] is the sub-list for method output_type
	5, // [5:9] is the sub-list for method input_type
	5, // [5:5] is the sub-list for extension type_name
	5, // [5:5] is the sub-list for extension extendee
	0, // [0:5] is the sub-list for field type_name
}

func init() { file_auth_auth_proto_init() }
func file_auth_auth_proto_init() {
	if File_auth_auth_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_auth_auth_proto_rawDesc), len(file_auth_auth_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   8,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_auth_auth_proto_goTypes,
		DependencyIndexes: file_auth_auth_proto_depIdxs,
		MessageInfos:      file_auth_auth_proto_msgTypes,
	}.Build()
	File_auth_auth_proto = out.File
	file_auth_auth_proto_goTypes = nil
	file_auth_auth_proto_depIdxs = nil
}
