// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// Status provides runtime information about the network such as the number of allocated IPs.
//
// swagger:model Status
type Status struct {

	// IPAM
	IPAM IPAMStatus `json:"IPAM"`
}
