// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// Inspect The body of the "get network" http response message.
//
// swagger:model Inspect
type Inspect struct {
	Network

	// Contains endpoints attached to the network.
	//
	// Example: {"19a4d5d687db25203351ed79d478946f861258f018fe384f229f2efa4b23513c":{"EndpointID":"628cadb8bcb92de107b2a1e516cbffe463e321f548feb37697cce00ad694f21a","IPv4Address":"172.19.0.2/16","IPv6Address":"","MacAddress":"02:42:ac:13:00:02","Name":"test"}}
	Containers map[string]EndpointResource `json:"Containers"`

	// List of services using the network. This field is only present for
	// swarm scope networks, and omitted for local scope networks.
	//
	Services map[string]ServiceInfo `json:"Services,omitempty"`

	// provides runtime information about the network such as the number of allocated IPs.
	//
	Status *Status `json:"Status,omitempty"`
}
