// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// IPAMStatus IPAM status
//
// swagger:model IPAMStatus
type IPAMStatus struct {

	// subnets
	// Example: {"172.16.0.0/16":{"DynamicIPsAvailable":65533,"IPsInUse":3},"2001:db8:abcd:0012::0/96":{"DynamicIPsAvailable":4294967291,"IPsInUse":5}}
	Subnets SubnetStatuses `json:"Subnets,omitempty"`
}
