// Generated automatically.  DO NOT HAND-EDIT.

package rxvt

import "github.com/gdamore/tcell/v2/terminfo"

func init() {

	// rxvt terminal emulator (X Window System)
	terminfo.AddTerminfo(&terminfo.Terminfo{
		Name:        "rxvt",
		Aliases:     []string{"rxvt-color"},
		Columns:     80,
		Lines:       24,
		Colors:      8,
		Clear:       "\x1b[H\x1b[2J",
		EnterCA:     "\x1b7\x1b[?47h",
		ExitCA:      "\x1b[2J\x1b[?47l\x1b8",
		ShowCursor:  "\x1b[?25h",
		HideCursor:  "\x1b[?25l",
		AttrOff:     "\x1b[m\x0f",
		Underline:   "\x1b[4m",
		Bold:        "\x1b[1m",
		Blink:       "\x1b[5m",
		Reverse:     "\x1b[7m",
		EnterKeypad: "\x1b=",
		ExitKeypad:  "\x1b>",
		SetFg:       "\x1b[3%p1%dm",
		SetBg:       "\x1b[4%p1%dm",
		SetFgBg:     "\x1b[3%p1%d;4%p2%dm",
		ResetFgBg:   "\x1b[39;49m",
		PadChar:     "\x00",
		AltChars:    "``aaffggjjkkllmmnnooppqqrrssttuuvvwwxxyyzz{{||}}~~",
		EnterAcs:    "\x0e",
		ExitAcs:     "\x0f",
		EnableAcs:   "\x1b(B\x1b)0",
		Mouse:       "\x1b[M",
		SetCursor:   "\x1b[%i%p1%d;%p2%dH",
		AutoMargin:  true,
		XTermLike:   true,
	})

	// rxvt 2.7.9 with xterm 256-colors
	terminfo.AddTerminfo(&terminfo.Terminfo{
		Name:        "rxvt-256color",
		Columns:     80,
		Lines:       24,
		Colors:      256,
		Clear:       "\x1b[H\x1b[2J",
		EnterCA:     "\x1b7\x1b[?47h",
		ExitCA:      "\x1b[2J\x1b[?47l\x1b8",
		ShowCursor:  "\x1b[?25h",
		HideCursor:  "\x1b[?25l",
		AttrOff:     "\x1b[m\x0f",
		Underline:   "\x1b[4m",
		Bold:        "\x1b[1m",
		Blink:       "\x1b[5m",
		Reverse:     "\x1b[7m",
		EnterKeypad: "\x1b=",
		ExitKeypad:  "\x1b>",
		SetFg:       "\x1b[%?%p1%{8}%<%t3%p1%d%e%p1%{16}%<%t9%p1%{8}%-%d%e38;5;%p1%d%;m",
		SetBg:       "\x1b[%?%p1%{8}%<%t4%p1%d%e%p1%{16}%<%t10%p1%{8}%-%d%e48;5;%p1%d%;m",
		SetFgBg:     "\x1b[%?%p1%{8}%<%t3%p1%d%e%p1%{16}%<%t9%p1%{8}%-%d%e38;5;%p1%d%;;%?%p2%{8}%<%t4%p2%d%e%p2%{16}%<%t10%p2%{8}%-%d%e48;5;%p2%d%;m",
		ResetFgBg:   "\x1b[39;49m",
		PadChar:     "\x00",
		AltChars:    "``aaffggjjkkllmmnnooppqqrrssttuuvvwwxxyyzz{{||}}~~",
		EnterAcs:    "\x0e",
		ExitAcs:     "\x0f",
		EnableAcs:   "\x1b(B\x1b)0",
		Mouse:       "\x1b[M",
		SetCursor:   "\x1b[%i%p1%d;%p2%dH",
		AutoMargin:  true,
		XTermLike:   true,
	})

	// rxvt 2.7.9 with xterm 88-colors
	terminfo.AddTerminfo(&terminfo.Terminfo{
		Name:        "rxvt-88color",
		Columns:     80,
		Lines:       24,
		Colors:      88,
		Clear:       "\x1b[H\x1b[2J",
		EnterCA:     "\x1b7\x1b[?47h",
		ExitCA:      "\x1b[2J\x1b[?47l\x1b8",
		ShowCursor:  "\x1b[?25h",
		HideCursor:  "\x1b[?25l",
		AttrOff:     "\x1b[m\x0f",
		Underline:   "\x1b[4m",
		Bold:        "\x1b[1m",
		Blink:       "\x1b[5m",
		Reverse:     "\x1b[7m",
		EnterKeypad: "\x1b=",
		ExitKeypad:  "\x1b>",
		SetFg:       "\x1b[%?%p1%{8}%<%t3%p1%d%e%p1%{16}%<%t9%p1%{8}%-%d%e38;5;%p1%d%;m",
		SetBg:       "\x1b[%?%p1%{8}%<%t4%p1%d%e%p1%{16}%<%t10%p1%{8}%-%d%e48;5;%p1%d%;m",
		SetFgBg:     "\x1b[%?%p1%{8}%<%t3%p1%d%e%p1%{16}%<%t9%p1%{8}%-%d%e38;5;%p1%d%;;%?%p2%{8}%<%t4%p2%d%e%p2%{16}%<%t10%p2%{8}%-%d%e48;5;%p2%d%;m",
		ResetFgBg:   "\x1b[39;49m",
		PadChar:     "\x00",
		AltChars:    "``aaffggjjkkllmmnnooppqqrrssttuuvvwwxxyyzz{{||}}~~",
		EnterAcs:    "\x0e",
		ExitAcs:     "\x0f",
		EnableAcs:   "\x1b(B\x1b)0",
		Mouse:       "\x1b[M",
		SetCursor:   "\x1b[%i%p1%d;%p2%dH",
		AutoMargin:  true,
		XTermLike:   true,
	})

	// rxvt-unicode terminal (X Window System)
	terminfo.AddTerminfo(&terminfo.Terminfo{
		Name:              "rxvt-unicode",
		Columns:           80,
		Lines:             24,
		Colors:            88,
		Clear:             "\x1b[H\x1b[2J",
		EnterCA:           "\x1b[?1049h",
		ExitCA:            "\x1b[r\x1b[?1049l",
		ShowCursor:        "\x1b[?12l\x1b[?25h",
		HideCursor:        "\x1b[?25l",
		AttrOff:           "\x1b[m\x1b(B",
		Underline:         "\x1b[4m",
		Bold:              "\x1b[1m",
		Italic:            "\x1b[3m",
		Blink:             "\x1b[5m",
		Reverse:           "\x1b[7m",
		EnterKeypad:       "\x1b=",
		ExitKeypad:        "\x1b>",
		SetFg:             "\x1b[38;5;%p1%dm",
		SetBg:             "\x1b[48;5;%p1%dm",
		SetFgBg:           "\x1b[38;5;%p1%d;48;5;%p2%dm",
		ResetFgBg:         "\x1b[39;49m",
		AltChars:          "+C,D-A.B0E``aaffgghFiGjjkkllmmnnooppqqrrssttuuvvwwxxyyzz{{||}}~~",
		EnterAcs:          "\x1b(0",
		ExitAcs:           "\x1b(B",
		EnableAutoMargin:  "\x1b[?7h",
		DisableAutoMargin: "\x1b[?7l",
		Mouse:             "\x1b[M",
		SetCursor:         "\x1b[%i%p1%d;%p2%dH",
		AutoMargin:        true,
		InsertChar:        "\x1b[@",
	})

	// rxvt-unicode terminal with 256 colors (X Window System)
	terminfo.AddTerminfo(&terminfo.Terminfo{
		Name:              "rxvt-unicode-256color",
		Columns:           80,
		Lines:             24,
		Colors:            256,
		Clear:             "\x1b[H\x1b[2J",
		EnterCA:           "\x1b[?1049h",
		ExitCA:            "\x1b[r\x1b[?1049l",
		ShowCursor:        "\x1b[?12l\x1b[?25h",
		HideCursor:        "\x1b[?25l",
		AttrOff:           "\x1b[m\x1b(B",
		Underline:         "\x1b[4m",
		Bold:              "\x1b[1m",
		Italic:            "\x1b[3m",
		Blink:             "\x1b[5m",
		Reverse:           "\x1b[7m",
		EnterKeypad:       "\x1b=",
		ExitKeypad:        "\x1b>",
		SetFg:             "\x1b[38;5;%p1%dm",
		SetBg:             "\x1b[48;5;%p1%dm",
		SetFgBg:           "\x1b[38;5;%p1%d;48;5;%p2%dm",
		ResetFgBg:         "\x1b[39;49m",
		AltChars:          "+C,D-A.B0E``aaffgghFiGjjkkllmmnnooppqqrrssttuuvvwwxxyyzz{{||}}~~",
		EnterAcs:          "\x1b(0",
		ExitAcs:           "\x1b(B",
		EnableAutoMargin:  "\x1b[?7h",
		DisableAutoMargin: "\x1b[?7l",
		Mouse:             "\x1b[M",
		SetCursor:         "\x1b[%i%p1%d;%p2%dH",
		AutoMargin:        true,
		InsertChar:        "\x1b[@",
	})
}
