// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11-devel
// 	protoc        (unknown)
// source: resource/config/config.proto

package config

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// MachineType matches machine.Type constants.
type MachineType int32

const (
	MachineType_UNKNOWN       MachineType = 0
	MachineType_INIT          MachineType = 1
	MachineType_CONTROL_PLANE MachineType = 2
	MachineType_WORKER        MachineType = 3
)

// Enum value maps for MachineType.
var (
	MachineType_name = map[int32]string{
		0: "UNKNOWN",
		1: "INIT",
		2: "CONTROL_PLANE",
		3: "WORKER",
	}
	MachineType_value = map[string]int32{
		"UNKNOWN":       0,
		"INIT":          1,
		"CONTROL_PLANE": 2,
		"WORKER":        3,
	}
)

func (x MachineType) Enum() *MachineType {
	p := new(MachineType)
	*p = x
	return p
}

func (x MachineType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (MachineType) Descriptor() protoreflect.EnumDescriptor {
	return file_resource_config_config_proto_enumTypes[0].Descriptor()
}

func (MachineType) Type() protoreflect.EnumType {
	return &file_resource_config_config_proto_enumTypes[0]
}

func (x MachineType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use MachineType.Descriptor instead.
func (MachineType) EnumDescriptor() ([]byte, []int) {
	return file_resource_config_config_proto_rawDescGZIP(), []int{0}
}

// MessageConfigSpec is the spec for the config.MachineConfig resource.
type MachineConfigSpec struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Contains YAML marshalled machine configuration.
	//
	// Byte representation is preserved as the machine configuration was submitted to the node.
	YamlMarshalled []byte `protobuf:"bytes,1,opt,name=yaml_marshalled,json=yamlMarshalled,proto3" json:"yaml_marshalled,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *MachineConfigSpec) Reset() {
	*x = MachineConfigSpec{}
	mi := &file_resource_config_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MachineConfigSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MachineConfigSpec) ProtoMessage() {}

func (x *MachineConfigSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_config_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MachineConfigSpec.ProtoReflect.Descriptor instead.
func (*MachineConfigSpec) Descriptor() ([]byte, []int) {
	return file_resource_config_config_proto_rawDescGZIP(), []int{0}
}

func (x *MachineConfigSpec) GetYamlMarshalled() []byte {
	if x != nil {
		return x.YamlMarshalled
	}
	return nil
}

// MachineTypeSpec is the spec for the config.MachineType resource.
type MachineTypeSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	MachineType   MachineType            `protobuf:"varint,1,opt,name=machine_type,json=machineType,proto3,enum=resource.config.MachineType" json:"machine_type,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MachineTypeSpec) Reset() {
	*x = MachineTypeSpec{}
	mi := &file_resource_config_config_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MachineTypeSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MachineTypeSpec) ProtoMessage() {}

func (x *MachineTypeSpec) ProtoReflect() protoreflect.Message {
	mi := &file_resource_config_config_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MachineTypeSpec.ProtoReflect.Descriptor instead.
func (*MachineTypeSpec) Descriptor() ([]byte, []int) {
	return file_resource_config_config_proto_rawDescGZIP(), []int{1}
}

func (x *MachineTypeSpec) GetMachineType() MachineType {
	if x != nil {
		return x.MachineType
	}
	return MachineType_UNKNOWN
}

var File_resource_config_config_proto protoreflect.FileDescriptor

const file_resource_config_config_proto_rawDesc = "" +
	"\n" +
	"\x1cresource/config/config.proto\x12\x0fresource.config\"<\n" +
	"\x11MachineConfigSpec\x12'\n" +
	"\x0fyaml_marshalled\x18\x01 \x01(\fR\x0eyamlMarshalled\"R\n" +
	"\x0fMachineTypeSpec\x12?\n" +
	"\fmachine_type\x18\x01 \x01(\x0e2\x1c.resource.config.MachineTypeR\vmachineType*C\n" +
	"\vMachineType\x12\v\n" +
	"\aUNKNOWN\x10\x00\x12\b\n" +
	"\x04INIT\x10\x01\x12\x11\n" +
	"\rCONTROL_PLANE\x10\x02\x12\n" +
	"\n" +
	"\x06WORKER\x10\x03B^\n" +
	"\x1ddev.talos.api.resource.configZ=github.com/siderolabs/talos/pkg/machinery/api/resource/configb\x06proto3"

var (
	file_resource_config_config_proto_rawDescOnce sync.Once
	file_resource_config_config_proto_rawDescData []byte
)

func file_resource_config_config_proto_rawDescGZIP() []byte {
	file_resource_config_config_proto_rawDescOnce.Do(func() {
		file_resource_config_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_resource_config_config_proto_rawDesc), len(file_resource_config_config_proto_rawDesc)))
	})
	return file_resource_config_config_proto_rawDescData
}

var file_resource_config_config_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_resource_config_config_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_resource_config_config_proto_goTypes = []any{
	(MachineType)(0),          // 0: resource.config.MachineType
	(*MachineConfigSpec)(nil), // 1: resource.config.MachineConfigSpec
	(*MachineTypeSpec)(nil),   // 2: resource.config.MachineTypeSpec
}
var file_resource_config_config_proto_depIdxs = []int32{
	0, // 0: resource.config.MachineTypeSpec.machine_type:type_name -> resource.config.MachineType
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_resource_config_config_proto_init() }
func file_resource_config_config_proto_init() {
	if File_resource_config_config_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_resource_config_config_proto_rawDesc), len(file_resource_config_config_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_resource_config_config_proto_goTypes,
		DependencyIndexes: file_resource_config_config_proto_depIdxs,
		EnumInfos:         file_resource_config_config_proto_enumTypes,
		MessageInfos:      file_resource_config_config_proto_msgTypes,
	}.Build()
	File_resource_config_config_proto = out.File
	file_resource_config_config_proto_goTypes = nil
	file_resource_config_config_proto_depIdxs = nil
}
