/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1alpha2

// This file contains a collection of methods that can be used from go-restful to
// generate Swagger API documentation for its models. Please read this PR for more
// information on the implementation: https://github.com/emicklei/go-restful/pull/215
//
// TODOs are ignored from the parser (e.g. TODO(andronat):... || TODO:...) if and only if
// they are on one line! For multiple line or blocks that you want to ignore use ---.
// Any context after a --- is ignored.
//
// Those methods can be generated by using hack/update-codegen.sh

// AUTO-GENERATED FUNCTIONS START HERE. DO NOT EDIT.
var map_BasicSchedulingPolicy = map[string]string{
	"": "BasicSchedulingPolicy indicates that standard Kubernetes scheduling behavior should be used.",
}

func (BasicSchedulingPolicy) SwaggerDoc() map[string]string {
	return map_BasicSchedulingPolicy
}

var map_GangSchedulingPolicy = map[string]string{
	"":         "GangSchedulingPolicy defines the parameters for gang scheduling.",
	"minCount": "MinCount is the minimum number of pods that must be schedulable or scheduled at the same time for the scheduler to admit the entire group. It must be a positive integer.",
}

func (GangSchedulingPolicy) SwaggerDoc() map[string]string {
	return map_GangSchedulingPolicy
}

var map_PodGroup = map[string]string{
	"":         "PodGroup represents a runtime instance of pods grouped together. PodGroups are created by workload controllers (Job, LWS, JobSet, etc...) from Workload.podGroupTemplates. PodGroup API enablement is toggled by the GenericWorkload feature gate.",
	"metadata": "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
	"spec":     "Spec defines the desired state of the PodGroup.",
	"status":   "Status represents the current observed state of the PodGroup.",
}

func (PodGroup) SwaggerDoc() map[string]string {
	return map_PodGroup
}

var map_PodGroupList = map[string]string{
	"":         "PodGroupList contains a list of PodGroup resources.",
	"metadata": "Standard list metadata.",
	"items":    "Items is the list of PodGroups.",
}

func (PodGroupList) SwaggerDoc() map[string]string {
	return map_PodGroupList
}

var map_PodGroupResourceClaim = map[string]string{
	"":                          "PodGroupResourceClaim references exactly one ResourceClaim, either directly or by naming a ResourceClaimTemplate which is then turned into a ResourceClaim for the PodGroup.\n\nIt adds a name to it that uniquely identifies the ResourceClaim inside the PodGroup. Pods that need access to the ResourceClaim define a matching reference in its own Spec.ResourceClaims. The Pod's claim must match all fields of the PodGroup's claim exactly.",
	"name":                      "Name uniquely identifies this resource claim inside the PodGroup. This must be a DNS_LABEL.",
	"resourceClaimName":         "ResourceClaimName is the name of a ResourceClaim object in the same namespace as this PodGroup. The ResourceClaim will be reserved for the PodGroup instead of its individual pods.\n\nExactly one of ResourceClaimName and ResourceClaimTemplateName must be set.",
	"resourceClaimTemplateName": "ResourceClaimTemplateName is the name of a ResourceClaimTemplate object in the same namespace as this PodGroup.\n\nThe template will be used to create a new ResourceClaim, which will be bound to this PodGroup. When this PodGroup is deleted, the ResourceClaim will also be deleted. The PodGroup name and resource name, along with a generated component, will be used to form a unique name for the ResourceClaim, which will be recorded in podgroup.status.resourceClaimStatuses.\n\nThis field is immutable and no changes will be made to the corresponding ResourceClaim by the control plane after creating the ResourceClaim.\n\nExactly one of ResourceClaimName and ResourceClaimTemplateName must be set.",
}

func (PodGroupResourceClaim) SwaggerDoc() map[string]string {
	return map_PodGroupResourceClaim
}

var map_PodGroupResourceClaimStatus = map[string]string{
	"":                  "PodGroupResourceClaimStatus is stored in the PodGroupStatus for each PodGroupResourceClaim which references a ResourceClaimTemplate. It stores the generated name for the corresponding ResourceClaim.",
	"name":              "Name uniquely identifies this resource claim inside the PodGroup. This must match the name of an entry in podgroup.spec.resourceClaims, which implies that the string must be a DNS_LABEL.",
	"resourceClaimName": "ResourceClaimName is the name of the ResourceClaim that was generated for the PodGroup in the namespace of the PodGroup. If this is unset, then generating a ResourceClaim was not necessary. The podgroup.spec.resourceClaims entry can be ignored in this case.",
}

func (PodGroupResourceClaimStatus) SwaggerDoc() map[string]string {
	return map_PodGroupResourceClaimStatus
}

var map_PodGroupSchedulingConstraints = map[string]string{
	"":         "PodGroupSchedulingConstraints defines scheduling constraints (e.g. topology) for a PodGroup.",
	"topology": "Topology defines the topology constraints for the pod group. Currently only a single topology constraint can be specified. This may change in the future.",
}

func (PodGroupSchedulingConstraints) SwaggerDoc() map[string]string {
	return map_PodGroupSchedulingConstraints
}

var map_PodGroupSchedulingPolicy = map[string]string{
	"":      "PodGroupSchedulingPolicy defines the scheduling configuration for a PodGroup. Exactly one policy must be set.",
	"basic": "Basic specifies that the pods in this group should be scheduled using standard Kubernetes scheduling behavior.",
	"gang":  "Gang specifies that the pods in this group should be scheduled using all-or-nothing semantics.",
}

func (PodGroupSchedulingPolicy) SwaggerDoc() map[string]string {
	return map_PodGroupSchedulingPolicy
}

var map_PodGroupSpec = map[string]string{
	"":                      "PodGroupSpec defines the desired state of a PodGroup.",
	"podGroupTemplateRef":   "PodGroupTemplateRef references an optional PodGroup template within other object (e.g. Workload) that was used to create the PodGroup. This field is immutable.",
	"schedulingPolicy":      "SchedulingPolicy defines the scheduling policy for this instance of the PodGroup. Controllers are expected to fill this field by copying it from a PodGroupTemplate. This field is immutable.",
	"schedulingConstraints": "SchedulingConstraints defines optional scheduling constraints (e.g. topology) for this PodGroup. Controllers are expected to fill this field by copying it from a PodGroupTemplate. This field is immutable. This field is only available when the TopologyAwareWorkloadScheduling feature gate is enabled.",
	"resourceClaims":        "ResourceClaims defines which ResourceClaims may be shared among Pods in the group. Pods consume the devices allocated to a PodGroup's claim by defining a claim in its own Spec.ResourceClaims that matches the PodGroup's claim exactly. The claim must have the same name and refer to the same ResourceClaim or ResourceClaimTemplate.\n\nThis is an alpha-level field and requires that the DRAWorkloadResourceClaims feature gate is enabled.\n\nThis field is immutable.",
	"disruptionMode":        "DisruptionMode defines the mode in which a given PodGroup can be disrupted. Controllers are expected to fill this field by copying it from a PodGroupTemplate. One of Pod, PodGroup. Defaults to Pod if unset. This field is immutable. This field is available only when the WorkloadAwarePreemption feature gate is enabled.",
	"priorityClassName":     "PriorityClassName defines the priority that should be considered when scheduling this pod group. Controllers are expected to fill this field by copying it from a PodGroupTemplate. Otherwise, it is validated and resolved similarly to the PriorityClassName on PodGroupTemplate (i.e. if no priority class is specified, admission control can set this to the global default priority class if it exists. Otherwise, the pod group's priority will be zero). This field is immutable. This field is available only when the WorkloadAwarePreemption feature gate is enabled.",
	"priority":              "Priority is the value of priority of this pod group. Various system components use this field to find the priority of the pod group. When Priority Admission Controller is enabled, it prevents users from setting this field. The admission controller populates this field from PriorityClassName. The higher the value, the higher the priority. This field is immutable. This field is available only when the WorkloadAwarePreemption feature gate is enabled.",
}

func (PodGroupSpec) SwaggerDoc() map[string]string {
	return map_PodGroupSpec
}

var map_PodGroupStatus = map[string]string{
	"":                      "PodGroupStatus represents information about the status of a pod group.",
	"conditions":            "Conditions represent the latest observations of the PodGroup's state.\n\nKnown condition types: - \"PodGroupScheduled\": Indicates whether the scheduling requirement has been satisfied. - \"DisruptionTarget\": Indicates whether the PodGroup is about to be terminated\n  due to disruption such as preemption.\n\nKnown reasons for the PodGroupScheduled condition: - \"Unschedulable\": The PodGroup cannot be scheduled due to resource constraints,\n  affinity/anti-affinity rules, or insufficient capacity for the gang.\n- \"SchedulerError\": The PodGroup cannot be scheduled due to some internal error\n  that happened during scheduling, for example due to nodeAffinity parsing errors.\n\nKnown reasons for the DisruptionTarget condition: - \"PreemptionByScheduler\": The PodGroup was preempted by the scheduler to make room for\n  higher-priority PodGroups or Pods.",
	"resourceClaimStatuses": "Status of resource claims.",
}

func (PodGroupStatus) SwaggerDoc() map[string]string {
	return map_PodGroupStatus
}

var map_PodGroupTemplate = map[string]string{
	"":                      "PodGroupTemplate represents a template for a set of pods with a scheduling policy.",
	"name":                  "Name is a unique identifier for the PodGroupTemplate within the Workload. It must be a DNS label. This field is immutable.",
	"schedulingPolicy":      "SchedulingPolicy defines the scheduling policy for this PodGroupTemplate.",
	"schedulingConstraints": "SchedulingConstraints defines optional scheduling constraints (e.g. topology) for this PodGroupTemplate. This field is only available when the TopologyAwareWorkloadScheduling feature gate is enabled.",
	"resourceClaims":        "ResourceClaims defines which ResourceClaims may be shared among Pods in the group. Pods consume the devices allocated to a PodGroup's claim by defining a claim in its own Spec.ResourceClaims that matches the PodGroup's claim exactly. The claim must have the same name and refer to the same ResourceClaim or ResourceClaimTemplate.\n\nThis is an alpha-level field and requires that the DRAWorkloadResourceClaims feature gate is enabled.\n\nThis field is immutable.",
	"disruptionMode":        "DisruptionMode defines the mode in which a given PodGroup can be disrupted. One of Pod, PodGroup. This field is available only when the WorkloadAwarePreemption feature gate is enabled.",
	"priorityClassName":     "PriorityClassName indicates the priority that should be considered when scheduling a pod group created from this template. If no priority class is specified, admission control can set this to the global default priority class if it exists. Otherwise, pod groups created from this template will have the priority set to zero. This field is available only when the WorkloadAwarePreemption feature gate is enabled.",
	"priority":              "Priority is the value of priority of pod groups created from this template. Various system components use this field to find the priority of the pod group. When Priority Admission Controller is enabled, it prevents users from setting this field. The admission controller populates this field from PriorityClassName. The higher the value, the higher the priority. This field is available only when the WorkloadAwarePreemption feature gate is enabled.",
}

func (PodGroupTemplate) SwaggerDoc() map[string]string {
	return map_PodGroupTemplate
}

var map_PodGroupTemplateReference = map[string]string{
	"":         "PodGroupTemplateReference references a PodGroup template defined in some object (e.g. Workload). Exactly one reference must be set.",
	"workload": "Workload references the PodGroupTemplate within the Workload object that was used to create the PodGroup.",
}

func (PodGroupTemplateReference) SwaggerDoc() map[string]string {
	return map_PodGroupTemplateReference
}

var map_TopologyConstraint = map[string]string{
	"":    "TopologyConstraint defines a topology constraint for a PodGroup.",
	"key": "Key specifies the key of the node label representing the topology domain. All pods within the PodGroup must be colocated within the same domain instance. Different PodGroups can land on different domain instances even if they derive from the same PodGroupTemplate. Examples: \"topology.kubernetes.io/rack\"",
}

func (TopologyConstraint) SwaggerDoc() map[string]string {
	return map_TopologyConstraint
}

var map_TypedLocalObjectReference = map[string]string{
	"":         "TypedLocalObjectReference allows to reference typed object inside the same namespace.",
	"apiGroup": "APIGroup is the group for the resource being referenced. If APIGroup is empty, the specified Kind must be in the core API group. For any other third-party types, setting APIGroup is required. It must be a DNS subdomain.",
	"kind":     "Kind is the type of resource being referenced. It must be a path segment name.",
	"name":     "Name is the name of resource being referenced. It must be a path segment name.",
}

func (TypedLocalObjectReference) SwaggerDoc() map[string]string {
	return map_TypedLocalObjectReference
}

var map_Workload = map[string]string{
	"":         "Workload allows for expressing scheduling constraints that should be used when managing the lifecycle of workloads from the scheduling perspective, including scheduling, preemption, eviction and other phases. Workload API enablement is toggled by the GenericWorkload feature gate.",
	"metadata": "Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
	"spec":     "Spec defines the desired behavior of a Workload.",
}

func (Workload) SwaggerDoc() map[string]string {
	return map_Workload
}

var map_WorkloadList = map[string]string{
	"":         "WorkloadList contains a list of Workload resources.",
	"metadata": "Standard list metadata.",
	"items":    "Items is the list of Workloads.",
}

func (WorkloadList) SwaggerDoc() map[string]string {
	return map_WorkloadList
}

var map_WorkloadPodGroupTemplateReference = map[string]string{
	"":                     "WorkloadPodGroupTemplateReference references the PodGroupTemplate within the Workload object.",
	"workloadName":         "WorkloadName defines the name of the Workload object.",
	"podGroupTemplateName": "PodGroupTemplateName defines the PodGroupTemplate name within the Workload object.",
}

func (WorkloadPodGroupTemplateReference) SwaggerDoc() map[string]string {
	return map_WorkloadPodGroupTemplateReference
}

var map_WorkloadSpec = map[string]string{
	"":                  "WorkloadSpec defines the desired state of a Workload.",
	"controllerRef":     "ControllerRef is an optional reference to the controlling object, such as a Deployment or Job. This field is intended for use by tools like CLIs to provide a link back to the original workload definition. This field is immutable.",
	"podGroupTemplates": "PodGroupTemplates is the list of templates that make up the Workload. The maximum number of templates is 8. This field is immutable.",
}

func (WorkloadSpec) SwaggerDoc() map[string]string {
	return map_WorkloadSpec
}

// AUTO-GENERATED FUNCTIONS END HERE
