// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"net/netip"
)

// ServiceInfo represents service parameters with the list of service's tasks
//
// swagger:model ServiceInfo
type ServiceInfo struct {

	// v IP
	VIP netip.Addr `json:"VIP"`

	// ports
	Ports []string `json:"Ports"`

	// local l b index
	LocalLBIndex int `json:"LocalLBIndex"`

	// tasks
	Tasks []Task `json:"Tasks"`
}
