// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// DisconnectRequest NetworkDisconnectRequest represents the data to be used to disconnect a container from a network.
//
// swagger:model DisconnectRequest
type DisconnectRequest struct {

	// The ID or name of the container to disconnect from the network.
	// Example: 3613f73ba0e4
	// Required: true
	Container string `json:"Container"`

	// Force the container to disconnect from the network.
	// Example: false
	Force bool `json:"Force"`
}
